// $Id: devicefactory.cc,v 1.45 2002/06/06 14:37:02 turbor Exp $

#include "config.h"

#include "MSXConfig.hh"
#include "devicefactory.hh"
#include "MSXSimple64KB.hh"
#include "MSXRom16KB.hh"
#include "MSXPPI.hh"
#include "VDP.hh"
#include "MSXE6Timer.hh"
#include "MSXCPU.hh"
#include "MSXMapperIO.hh"
#include "MSXPSG.hh"
#include "MSXMusic.hh"
#include "MSXFmPac.hh"
#include "MSXAudio.hh"
#include "MC6850.hh"
#include "MSXKanji.hh"
#include "MSXMemoryMapper.hh"
#include "MSXRTC.hh"
#include "MSXGameCartridge.hh"
#include "MSXPrinterPort.hh"
#include "MSXSCCPlusCart.hh"
#include "MSXARCdebug.hh"
#include "MSXFDC.hh"
//#include "MSXPrinter.hh"
//#include "MSXPostLoad.hh"


MSXDevice *deviceFactory::create(MSXConfig::Device *conf, const EmuTime &time) {
	MSXDevice *device = NULL;
	if (conf->getType()=="CPU") {
		device = MSXCPU::instance();
	} else
	if (conf->getType()=="MapperIO") {
		device = MSXMapperIO::instance();
	} else
	if (conf->getType()=="PPI") {
		device = new MSXPPI(conf, time);
	} else
	if (conf->getType()=="Rom16KB") {
		device = new MSXRom16KB(conf, time);
	} else
	if (conf->getType()=="Simple64KB") {
		device = new MSXSimple64KB(conf, time);
	} else
	if (conf->getType()=="VDP") {
		device = new VDP(conf, time);
	} else
	if (conf->getType()=="E6Timer") {
		device = new MSXE6Timer(conf, time);
	} else
	if (conf->getType()=="PSG") {
		device = new MSXPSG(conf, time);
	} else
#ifndef DONT_WANT_MSXMUSIC
	if (conf->getType()=="Music") {
		device = new MSXMusic(conf, time);
	} else
#endif
#ifndef DONT_WANT_FMPAC
	if (conf->getType()=="FM-PAC") {
		device = new MSXFmPac(conf, time);
	} else
#endif
	if (conf->getType()=="Audio") {
		device = new MSXAudio(conf, time);
	} else
	if (conf->getType()=="Audio-Midi") {
		device = new MC6850(conf, time);
	} else
	if (conf->getType()=="Kanji") {
		device = new MSXKanji(conf, time);
	} else
	if (conf->getType()=="MemoryMapper") {
		device = new MSXMemoryMapper(conf, time);
	} else
	if (conf->getType()=="RTC") {
		device = new MSXRTC(conf, time);
	} else
	if (conf->getType()=="GameCartridge") {
		device = new MSXGameCartridge(conf, time);
	} else
	if (conf->getType()=="PrinterPort") {
		device = new MSXPrinterPort(conf, time);
	} else
#ifndef DONT_WANT_SCC
	if (conf->getType()=="SCCPlusCart") {
		device = new MSXSCCPlusCart(conf, time);
	} else
#endif
	if (conf->getType()=="ARCdebug") {
		device = new MSXARCdebug(conf, time);
	} else 
	if (conf->getType()=="FDC") {
		device = new MSXFDC(conf, time);
	}
//	} else
//	if (conf->getType()=="PostLoad") {
//		device = new MSXPostLoad(conf, time);
//	}
	if (device == NULL)
		PRT_ERROR("Unknown device specified in configuration");
	return device;
}

